c
c*******************************************************
c
c	Test for the matrix sweaping (complex variables case)
C               Zemlyanaya E.V. 
c
c*******************************************************
c
      implicit real*8 (a-h,p-z)
	implicit complex*16 (o)
      parameter (n=101)
      parameter (l=2)

      dimension oa(l,l,n),ob(l,l,n),oc(l,l,n),od(l,n),oz(l,n),
     ^	x(n),oaa(l,l,n),obb(l,n),ozan(l,n)
c
c
c
c
c
c--------------------------TEST	(system of two diff equations)
c
c
c	u''(x) - u(x)/(x*x) + v(x) = i*(x*x-1/x)
c	v''(x) - v(x)/(x*x) - u(x) = i*(1-x)
c
c	
c
c	analytic solution (ozan): u(x) = i*x; v(x) = i*x^2
c
c	u(x_i) -->  oz(1,i)
c	v(x_i) -->  oz(2,i)
c
c	the second order finite difference approximation 
c	has been applied; the stepsoze is h
c
c----------------------------------------------------------------
c

	oi = (0.d0,1.d0)

	rmin = 1.d0
	rmax = 2.d0
	
	nm1 = n-1
	nm2 = n-2

	h = (rmax-rmin)/nm1
	hh = h*h	
 
	do i=1,n

	x(i) = rmin+(i-1)*h
	ozan(1,i) = x(i)*oi
	ozan(2,i) = x(i)*x(i)*oi
	end do

c--------------------------------------------------------
	
	do i=2,nm1

	xx = x(i)*x(i)

	ip1 = i+1
	im1 = i-1

      oa(1,1,i) = 1.d0/hh
	oa(1,2,i) = 0.d0
	oa(2,1,i) = 0.d0
      oa(2,2,i) = 1.d0/hh

	oc(1,1,i) = oa(1,1,i)
	oc(1,2,i) = oa(1,2,i)
	oc(2,1,i) = oa(2,1,i)
	oc(2,2,i) = oa(2,2,i) 

	ob(1,1,i) = -2.d0/hh -1.d0/xx 
	ob(1,2,i) = +1.d0
	ob(2,1,i) = -1.d0
	ob(2,2,i) = -2.d0/hh -1.d0/xx


	od(1,i) = (xx-1.d0/x(i))*oi
	od(2,i) = (1.d0-x(i))*oi
		
	end do

	oa(1,1,1) = 1.d0
	oa(1,2,1) = 0.0d0
	oa(2,1,1) = 0.0d0
	oa(2,2,1) = 1.d0
	ob(1,1,1) = 0.0d0
	ob(1,2,1) = 0.0d0
	ob(2,1,1) = 0.0d0
	ob(2,2,1) = 0.0d0
	oc(1,1,1) = 0.0d0
	oc(1,2,1) = 0.0d0
	oc(2,1,1) = 0.0d0
	oc(2,2,1) = 0.0d0
	od(1,1)   = ozan(1,1)
	od(2,1)   = ozan(2,1)

	oa(1,1,n) = 0.0d0
 	oa(1,2,n) = 0.0d0
	oa(2,1,n) = 0.0d0
	oa(2,2,n) = 0.0d0
	ob(1,1,n) = 0.0d0
	ob(1,2,n) = 0.0d0
	ob(2,1,n) = 0.0d0
	ob(2,2,n) = 0.0d0
	oc(1,1,n) = 1.d0
	oc(1,2,n) = 0.0d0
	oc(2,1,n) = 0.0d0
	oc(2,2,n) = 1.d0
	od(1,n)   = ozan(1,n)
	od(2,n)   = ozan(2,n)

c
        call cmatprog(n,l,oa,ob,oc,od,oz,oaa,obb)
c

c----------------------------------- checking residual 
c
	odelmax = 0.
	do  k=1,l
        i=1
	odel = oa(k,1,1)*oz(1,1)+oa(k,2,1)*oz(2,1)+ob(k,1,1)*oz(1,2)+
     ^  ob(k,2,1)*oz(2,2)+oc(k,1,1)*oz(1,3)+oc(k,2,1)*oz(2,3)-od(k,1)
	write (1,*) k,i,abs(odel)
c
      do i=2,nm1
c
       odel = oa(k,1,i)*oz(1,i-1)+oa(k,2,i)*oz(2,i-1)+ob(k,1,i)*oz(1,i)+
     ^ ob(k,2,i)*oz(2,i)+oc(k,1,i)*oz(1,i+1)+oc(k,2,i)*oz(2,i+1)-od(k,i)
	write (1,*) k,i,abs(odel)
      end do

        i = n
      odel=oa(k,1,n)*oz(1,nm2)+oa(k,2,n)*oz(2,nm2)+ob(k,1,n)*oz(1,nm1)+
     ^  ob(k,2,n)*oz(2,nm1)+oc(k,1,n)*oz(1,n)+oc(k,2,n)*oz(2,n)-od(k,n)
	write (1,*) k,i,abs(odel)
      end do	

C--------------------- COMPARISON WITH THE ANALYTIC SOLUTI0ON   

        do i=1,n
        write (11,*) x(i),abs(oz(1,i)-ozan(1,i)),abs(oz(2,i)-ozan(2,i))
        end do

       END



c***********************************************************
c*
      subroutine cmatprog(n,l,oa,ob,oc,od,oz,oaa,obb)
c
c*********************************************************
c
c              Zemlyanaya E.V. COMPLEX VARIABLES CASE
c
c              Matrix sweaping for the algebraic system  
c
c              a_i*z_{i-1}+b_i*z_i+c_i*z_{i+1} = d_i
c			
c			 where i=1,...,n; 	
c              a,b,c are matrixes of coefficients (l,l,n);
c			 d is vector of right hand (l,n); 
c			 z is vector of solution (l,n) - OUTPUT.
c
c			
c*********************************************************
c
      implicit real*8 (a-h,p-z)
	implicit complex*16 (o)
      dimension oa(l,l,n),ob(l,l,n),oc(l,l,n),od(l,n),oz(l,n)
      dimension oaa(l,l,n),obb(l,n)

      dimension or1(l,l),or2(l,l),or3(l,l),or4(l,l),or5(l,l),or6(l,l),
     ^	or7(l,l),ov1(l),ov2(l),ov3(l),ov4(l),ov5(l)

ccc      dimension or1(2,2),or2(2,2),or3(2,2),or4(2,2),or5(2,2),
ccc     ^	or6(2,2),or7(2,2)
ccc      dimension ov1(2),ov2(2),ov3(2),ov4(2),ov5(2)

        nm1  = n-1
        nm2  = n-2
c
c--------------------------------------- left system
c
      do i = 1,l
      do j = 1,l
        or1(i,j) = oa(i,j,1)
        or2(i,j) = ob(i,j,1)
        or3(i,j) = oa(i,j,2)
        or4(i,j) = ob(i,j,2)
      end do
        ov1(i) = od(i,1)
        ov2(i) = od(i,2)
      end do
c
c--------------------------------(a2*b1-a1*b2)^{-1} -> r7
c
        call multmmc(l,or3,or2,or5)
        call multmmc(l,or1,or4,or6)

      do i=1,l
      do j=1,l
        or7(i,j) = or5(i,j)-or6(i,j)
      end do
      end do

        call inverse_c(l,or7)

c
c-------------------------------(a2*c1-a1*c2) -> r5
c
      do i=1,l
      do j=1,l
        or2(i,j) = oc(i,j,1)
        or4(i,j) = oc(i,j,2)
      end do
      end do

        call multmmc(l,or3,or2,or5)
        call multmmc(l,or1,or4,or6)

      do i=1,l
      do j=1,l
        or5(i,j) = or5(i,j) - or6(i,j)
      end do
      end do
c
c------------------------------(a2*d1-a1*d2) -> v5
c
        call multmvc(l,or3,ov1,ov3)
        call multmvc(l,or1,ov2,ov4)

      do i=1,l
        ov5(i) = ov3(i) - ov4(i)
      end do

        call multmmc(l,or7,or5,or6)
        call multmvc(l,or7,ov5,ov4)

c
c---------------------------------- -r6->A_2, v4->B_2
c
      do i=1,l
      do j=1,l
        or6(i,j)    = -or6(i,j)
        oaa(i,j,2) = or6(i,j)
      end do
        obb(i,2)   = ov4(i)
      end do
c
c----------------------------------- k=3,N-1
c

      do k = 3,nm1

        km1 = k-1
      do i = 1,l
      do j = 1,l
        or1(i,j) = oa(i,j,k)
        or2(i,j) = ob(i,j,k)
        or3(i,j) = oc(i,j,k)
      end do
        ov1(i) = od(i,k)
      end do
c
c-----------------------------(bi+ai*A_{i-1}) -> r7
c-----------------------------(di-ai*B_{i-1}) -> v2
c-----------------------------(A_{i-1}*ci)    -> r5
c
        call multmmc(l,or1,or6,or4)
        call multmvc(l,or1,ov4,ov2)

      do i=1,l
      do j=1,l
        or7(i,j) = or2(i,j)+or4(i,j)
      end do
        ov3(i) = ov1(i)-ov2(i)
      end do

        call inverse_c(l,or7)
c
c--------------------------------Ai -> -r6;  Bi -> v4
c
        call multmmc(l,or7,or3,or6)
        call multmvc(l,or7,ov3,ov4)

      do i=1,l
      do j=1,l
        or6(i,j)  = -or6(i,j)
        oaa(i,j,k) = or6(i,j)
      end do
        obb(i,k) = ov4(i)
      end do

      end do
c
c--------------------------------------- right system
c
      do i = 1,l
      do j = 1,l
        or1(i,j) = oa(i,j,n)
        or2(i,j) = ob(i,j,n)
        or3(i,j) = oc(i,j,n)
	  or6(i,j) = oaa(i,j,nm2)
      end do
      ov1(i) = od(i,n)
	ov4(i)  = obb(i,nm2)
      end do
c
     	call multmmc(l,or1,or6,or4)
	call multmvc(l,or1,ov4,ov2)
		
      do i=1,l
      do j=1,l
	or4(i,j) = or4(i,j)+or2(i,j)
	or6(i,j) = oaa(i,j,nm1)
      end do
	ov4(i) = obb(i,nm1)
      end do

	call multmmc(l,or4,or6,or5)
	call multmvc(l,or4,ov4,ov5)

      do i=1,l
      do j=1,l
	or5(i,j) = or5(i,j)+or3(i,j)
      end do
	ov5(i) = ov1(i)-ov2(i)-ov5(i)
      end do

	call inverse_c(l,or5)
	call multmvc(l,or5,ov5,ov4)

      do i=1,l
        oz(i,n) = ov4(i)
      end do

c
c------------------------------------ the reverse 'progonka'
c
      do k = nm1,2,-1
       kp1 = k+1
      do i = 1,l
      do j = 1,l
       or1(i,j)   = oaa(i,j,k)
      end do
        ov1(i) = oz(i,kp1)
      end do

        call multmvc(l,or1,ov1,ov2)

      do i=1,l
        oz(i,k) = ov2(i)+obb(i,k)
      end do
      end do
c
c---------------------------------------- z_1= a_2^{-1}*[d_2-b_2*z_2-c_2*z_3]
c
      do i=1,l
      do j=1,l

        or1(i,j) = oa(i,j,2)
	  or2(i,j) = ob(i,j,2)
	  or3(i,j) = oc(i,j,2)
      end do
	  ov1(i) = od(i,2)
	  ov2(i) = oz(i,2)
	  ov3(i) = oz(i,3)
      end do

	  call inverse_c(l,or1)
	  call multmvc(l,or2,ov2,ov4)
	  call multmvc(l,or3,ov3,ov5)

      do i=1,l
	  ov5(i) = ov1(i) - ov4(i) - ov5(i)
      end do

	  call multmvc(l,or1,ov5,ov1)

      do i=1,l
	  oz(i,1) = ov1(i)
      end do

	  return
        end
c
c*******************************************************************
c
      SUBROUTINE  MATIN2 (A,idim1,N1,idim2,N2,INDEX,NERROR,DETERM)
c     implicit real*8 (a-h,o-z)
C	 MATRIX INVERSION WITH ACCOMPANYING SOLUTION OF LINEAR EQUATIONS
      DOUBLE PRECISION A,DETERM,DETER,PIVOT,SWAP
      DIMENSION A(*),INDEX(*)
	idim2 = idim1
      DETER=1.0D0
      N=N1
      IEMAT=N+N2
      IDIM=IDIM1
      NMIN1=N-1
C	 THE ROUTINE DOES ITS OWN EVALUATION FOR DOUBLE SUBSCRIPTING OF
C	 ARRAY A.
      IPIVC=1-IDIM
C	 MAIN LOOP TO INVERT THE MATRIX
      DO 11 MAIN=1,N
      PIVOT=0.0D0
      IPIVC=IPIVC+IDIM
C	 SEARCH FOR NEXT PIVOT IN COLUMN MAIN.
      IPIVC1=IPIVC+MAIN-1
      IPIVC2=IPIVC +NMIN1
      DO 2 I1=IPIVC1,IPIVC2
      IF(DABS(A(I1))-DABS(PIVOT)) 2,2,1
    1 PIVOT=A(I1)
      LPIV=I1
    2 CONTINUE
C	 IS PIVOT DIFFERENT FROM ZERO
      IF(PIVOT) 3,15,3
C	 GET THE PIVOT-LINE INDICATOR AND SWAP LINES IF NECESSARY
    3 ICOL=LPIV-IPIVC+1
      INDEX(MAIN)=ICOL
      IF(ICOL-MAIN) 6,6,4
C	 COMPLEMENT THE DETERMINANT
    4 DETER=-DETER
C	 POINTER TO LINE PIVOT FOUND
      ICOL=ICOL-IDIM
C	 POINTER TO EXACT PIVOT LINE
      I3=MAIN-IDIM
      DO 5 I=1,IEMAT
      ICOL=ICOL+IDIM
      I3=I3+IDIM
      SWAP=A(I3)
      A(I3)=A(ICOL)
    5 A(ICOL)=SWAP
C	 COMPUTE DETERMINANT
    6 DETER=DETER*PIVOT
      PIVOT=1./PIVOT
C	 TRANSFORM PIVOT COLUMN
      I3=IPIVC+NMIN1
      DO 7 I=IPIVC,I3
    7 A(I)=-A(I)*PIVOT
      A(IPIVC1)=PIVOT
C	 PIVOT ELEMENT TRANSFORMED
C
C	 NOW CONVERT REST OF THE MATRIX
      I1=MAIN-IDIM
C	 POINTER TO PIVOT LINE ELEMENTS
      ICOL=1-IDIM
C	 GENERAL COLUMN POINTER
      DO 10 I=1,IEMAT
      ICOL=ICOL+IDIM
      I1=I1+IDIM
C	 POINTERS MOVED
      IF(I-MAIN) 8,10,8
C	 PIVOT COLUMN EXCLUDED
    8 JCOL=ICOL+NMIN1
      SWAP=A(I1)
      I3=IPIVC-1
      DO 9 I2=ICOL,JCOL
      I3=I3+1
    9 A(I2)=A(I2)+SWAP*A(I3)
      A(I1)=SWAP*PIVOT
   10 CONTINUE
   11 CONTINUE
C	 NOW REARRANGE THE MATRIX TO GET RIGHT INVERS
      DO 14 I1=1,N
      MAIN=N+1-I1
      LPIV=INDEX(MAIN)
      IF(LPIV-MAIN) 12,14,12
   12 ICOL=(LPIV-1)*IDIM+1
      JCOL=ICOL+NMIN1
      IPIVC=(MAIN-1)*IDIM+1-ICOL
      DO 13 I2=ICOL,JCOL
      I3=I2+IPIVC
      SWAP=A(I2)
      A(I2)=A(I3)
   13 A(I3)=SWAP
   14 CONTINUE
      DETERM=DETER
      NERROR=0
      RETURN
   15 NERROR=MAIN
      DETERM=DETER
      RETURN
      END


c*
c***************************************************************
c*
 
c*
c*******************************************************************
c
      subroutine inverse_c(n,oa)
      implicit real*8 (a-h,p-z)
      implicit complex*16 (o)
      dimension oa(n,n)
      dimension ar(n,n),ai(n,n),br(n,n),bi(n,n),r1(n,n),r2(n,n),r3(n,n)
	dimension rsum(n,n),rmin(n,n)
      dimension index(n)
c 
      DO  I  = 1,n
      do  j  = 1,n

      ar(I,J) = real (oa(i,j))
      ai(i,j) = imag(oa(i,j))
	r3(i,j) = 0.d0
	rmin(i,j) = ar(i,j)-ai(i,j)
	rsum(i,j) = ar(i,j)+ai(i,j)
	end do
      r3(i,i) = ai(i,i)
	end do
      
      call multmm(n,ai,ai,r1)
      call multmm(n,ar,ar,r2)

      do i    = 1,n
      do j    = 1,n
      r2(i,j) = r1(i,j)-r2(i,j)
      end do
      end do

      call matin2(r2,n,n,mdim,0,index,nerror,determ)
	call multmm(n,r2,r3,bi)

      call matin2(rsum,n,n,mdim,0,index,nerror,determ)
      call multmm(n,rmin,bi,r1)
      do i    = 1,n
	r1(i,i) = r1(i,i)+1.d0
	end do
      call multmm(n,rsum,r1,br)
      DO 1 I  = 1,N
      DO 1 J  = 1,N
      oa(i,j) = cmplx(br(i,j),bi(i,j))
  1   CONTINUE
c
      RETURN
      END
C
C*******************************************************************
C
      SUBROUTINE MULTMV(N,A,B,C)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(N,N),B(N),C(N)
      DO 1 I = 1,N
      C(I)   = 0.d0
      DO 1 J = 1,N
      C(I)   = C(I)+A(I,J)*B(J)
  1   CONTINUE
      RETURN
      END
C
C*******************************************************************
C
      SUBROUTINE MULTMM(N,A,B,C)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(N,N),B(N,N),C(N,N)
      DO 2 I = 1,N
      DO 2 J = 1,N
      C(I,J) = 0.d0
      DO 2 L = 1,N
      C(I,J) = C(I,J)+A(I,L)*B(L,J)
  2   CONTINUE
      RETURN
      END
c*
C
C*******************************************************************
C
      SUBROUTINE MULTMVC(N,OA,OB,OC)
      IMPLICIT REAL*8 (A-H,P-Z)
      implicit complex*16 (o)
      DIMENSION OA(N,N),OB(N),OC(N)
      DO 1 I = 1,N
      OC(I)   = 0.d0
      DO 1 J = 1,N
      OC(I)   = OC(I)+OA(I,J)*OB(J)
  1   CONTINUE
      RETURN
      END
C
C*******************************************************************
C
      SUBROUTINE MULTMMC(N,OA,OB,OC)
      IMPLICIT REAL*8 (A-H,P-Z)
	implicit complex*16 (o)
      DIMENSION OA(N,N),OB(N,N),OC(N,N)
      DO 2 I = 1,N
      DO 2 J = 1,N
      OC(I,J) = 0.d0
      DO 2 L = 1,N
      OC(I,J) = OC(I,J)+OA(I,L)*OB(L,J)
  2   CONTINUE
      RETURN
      END
c*

