c
c*******************************************************
c
c	Test for the matrix sweaping (real variables)
C               Zemlyanaya E.V. 
c
c*******************************************************
c
      implicit real*8 (a-h,o-z)
      parameter (n=101)
      parameter (l=2)

      dimension a(l,l,n),b(l,l,n),c(l,l,n),d(l,n),z(l,n),
     ^	x(n),aa(l,l,n),bb(l,l,n),zan(l,n)
c
c--------------------------TEST (system of two diff equations)

c		z1''+w11*z1+w12*z2=0
c		z2''+w21*z1+w22*z2=0
c
c		zan - analytic solution; x - argument.
c		we use the 2nd order finite-difference approximation
c		with the stepsoze h.
c
c	w11 = -2; w12 = 3; w21 = -1; w22 =2.

	rmin = 1.d0
	rmax = 2.d0
	
	nm1 = n-1
	nm2 = n-2
	h = (rmax-rmin)/nm1
	hh = h*h	
 
	do i=1,n
	x(i) = rmin+(i-1)*h
	zan(1,i) = 3.d0*exp(-x(i))
	zan(2,i) =  exp(-x(i))
	end do
	
	do i=2,nm1
	ip1 = i+1
	im1 = i-1
      a(1,1,i) = 1.d0/hh
	a(1,2,i) = 0.d0
	a(2,1,i) = 0.d0
      a(2,2,i) = 1.d0/hh

	c(1,1,i) = a(1,1,i)
	c(1,2,i) = a(1,2,i)
	c(2,1,i) = a(2,1,i)
	c(2,2,i) = a(2,2,i)

	b(1,1,i) = -2.d0/hh -2.d0
	b(1,2,i) = +3.d0
	b(2,1,i) = -1.d0
	b(2,2,i) = -2.d0/hh +2.d0
	
	end do

	a(1,1,1) = 1.d0
	a(1,2,1) = 0.0d0
	a(2,1,1) = 0.0d0
	a(2,2,1) = 1.d0
	b(1,1,1) = 0.0d0
	b(1,2,1) = 0.0d0
	b(2,1,1) = 0.0d0
	b(2,2,1) = 0.0d0
	c(1,1,1) = 0.0d0
	c(1,2,1) = 0.0d0
	c(2,1,1) = 0.0d0
	c(2,2,1) = 0.0d0
	d(1,1)   = zan(1,1)
	d(2,1)   = zan(2,1)

	a(1,1,n) = 0.0d0
 	a(1,2,n) = 0.0d0
	a(2,1,n) = 0.0d0
	a(2,2,n) = 0.0d0
	b(1,1,n) = 0.0d0
	b(1,2,n) = 0.0d0
	b(2,1,n) = 0.0d0
	b(2,2,n) = 0.0d0
	c(1,1,n) = 1.d0
	c(1,2,n) = 0.0d0
	c(2,1,n) = 0.0d0
	c(2,2,n) = 1.d0
	d(1,n)   = zan(1,n)
	d(2,n)   = zan(2,n)

c
        call matprog(n,l,a,b,c,d,z,aa,bb)
c
c----------------------------------- checking residual
c
	delmax = 0.
	do  k=1,l
        i=1
	del = a(k,1,1)*z(1,1)+a(k,2,1)*z(2,1)+b(k,1,1)*z(1,2)+
     ^  b(k,2,1)*z(2,2)+c(k,1,1)*z(1,3)+c(k,2,1)*z(2,3)-d(k,1)
	write (1,*) k,i,del,z(k,i)
	del = abs(del)
      if(del.ge.delmax) then
	delmax = del
	lmax = k
	imax = i
      end if
c
      do i=2,nm1
c
        del = a(k,1,i)*z(1,i-1)+a(k,2,i)*z(2,i-1)+b(k,1,i)*z(1,i)+
     ^  b(k,2,i)*z(2,i)+c(k,1,i)*z(1,i+1)+c(k,2,i)*z(2,i+1)-d(k,i)
	write (1,*) k,i,del,z(k,i)
	if(del.ge.delmax) then
	delmax = del
	lmax = k
	imax = i
      end if
      end do

        i = n
        del = a(k,1,n)*z(1,nm2)+a(k,2,n)*z(2,nm2)+b(k,1,n)*z(1,nm1)+
     ^  b(k,2,n)*z(2,nm1)+c(k,1,n)*z(1,n)+c(k,2,n)*z(2,n)-d(k,n)
	write (1,*) k,i,del,z(k,i)
	del = abs(del)
      if(del.ge.delmax) then
	delmax = del
	lmax = k
	imax = i
      end if
      end do	

	print *,'maximal residual',delmax,' is for l,i:',lmax,imax

c
c----------------- comparizon with analytic solution
c
       
        do i=1,n
        write (11,1111) x(i),z(1,i)-zan(1,i),z(2,i)-zan(2,i)
        end do


1111   format(5(x,e15.9))
2000   format(f10.5,10(x,e13.7))

       END

c***********************************************************
c***********************************************************
c*
      subroutine matprog(n,l,a,b,c,d,z,aa,bb)
c
c*********************************************************
c
c              Zemlyanaya E.V. PURE REAL VARIABLES CASE
c
c              Matrix sweaping for the algebraic system  
c
c              a_i*z_{i-1}+b_i*z_i+c_i*z_{i+1} = d_i
c			
c			 where i=1,...,n; 	
c              a,b,c are matrixes of coefficients (l,l,n);
c			 d is vector of right hand (l,n); 
c			 z is vector of solution (l,n) - OUTPUT.
c
c*********************************************************
c
      implicit real*8 (a-h,o-z)
      dimension a(l,l,n),b(l,l,n),c(l,l,n),d(l,n),z(l,n)
      dimension aa(l,l,n),bb(l,n)

      dimension r1(l,l),r2(l,l),r3(l,l),r4(l,l),r5(l,l),r6(l,l),r7(l,l)
      dimension v1(l),v2(l),v3(l),v4(l),v5(l)

ccc      dimension r1(2,2),r2(2,2),r3(2,2),r4(2,2),r5(2,2),r6(2,2),r7(2,2)
ccc      dimension v1(2),v2(2),v3(2),v4(2),v5(2)

        nm1  = n-1
        nm2  = n-2
c
c--------------------------------------- left system
c
      do i = 1,l
      do j = 1,l
        r1(i,j) = a(i,j,1)
        r2(i,j) = b(i,j,1)
        r3(i,j) = a(i,j,2)
        r4(i,j) = b(i,j,2)
      end do
        v1(i) = d(i,1)
        v2(i) = d(i,2)
      end do
c
c--------------------------------(a2*b1-a1*b2)^{-1} -> r7
c
        call multmm(l,r3,r2,r5)
        call multmm(l,r1,r4,r6)

      do i=1,l
      do j=1,l
        r7(i,j) = r5(i,j)-r6(i,j)
      end do
      end do

        call inverse(l,r7)

c
c-------------------------------(a2*c1-a1*c2) -> r5
c
      do i=1,l
      do j=1,l
        r2(i,j) = c(i,j,1)
        r4(i,j) = c(i,j,2)
      end do
      end do

        call multmm(l,r3,r2,r5)
        call multmm(l,r1,r4,r6)

      do i=1,l
      do j=1,l
        r5(i,j) = r5(i,j) - r6(i,j)
      end do
      end do
c
c------------------------------(a2*d1-a1*d2) -> v5
c
        call multmv(l,r3,v1,v3)
        call multmv(l,r1,v2,v4)

      do i=1,l
        v5(i) = v3(i) - v4(i)
      end do

        call multmm(l,r7,r5,r6)
        call multmv(l,r7,v5,v4)

c
c---------------------------------- -r6->A_2, v4->B_2
c
      do i=1,l
      do j=1,l
        r6(i,j)    = -r6(i,j)
        aa(i,j,2) = r6(i,j)
      end do
        bb(i,2)   = v4(i)
      end do
c
c----------------------------------- k=3,N-1
c

      do k = 3,nm1

        km1 = k-1
      do i = 1,l
      do j = 1,l
        r1(i,j) = a(i,j,k)
        r2(i,j) = b(i,j,k)
        r3(i,j) = c(i,j,k)
      end do
        v1(i) = d(i,k)
      end do
c
c-----------------------------(bi+ai*A_{i-1}) -> r7
c-----------------------------(di-ai*B_{i-1}) -> v2
c-----------------------------(A_{i-1}*ci)    -> r5
c
        call multmm(l,r1,r6,r4)
        call multmv(l,r1,v4,v2)

      do i=1,l
      do j=1,l
        r7(i,j) = r2(i,j)+r4(i,j)
      end do
        v3(i) = v1(i)-v2(i)
      end do

        call inverse(l,r7)
c
c--------------------------------Ai -> -r6;  Bi -> v4
c
        call multmm(l,r7,r3,r6)
        call multmv(l,r7,v3,v4)

      do i=1,l
      do j=1,l
        r6(i,j)  = -r6(i,j)
        aa(i,j,k) = r6(i,j)
      end do
        bb(i,k) = v4(i)
      end do

      end do
c
c--------------------------------------- right system
c
      do i = 1,l
      do j = 1,l
        r1(i,j) = a(i,j,n)
        r2(i,j) = b(i,j,n)
        r3(i,j) = c(i,j,n)
	  r6(i,j) = aa(i,j,nm2)
      end do
      v1(i) = d(i,n)
	v4(i)  = bb(i,nm2)
      end do
c
     	call multmm(l,r1,r6,r4)
	call multmv(l,r1,v4,v2)
		
      do i=1,l
      do j=1,l
	r4(i,j) = r4(i,j)+r2(i,j)
	r6(i,j) = aa(i,j,nm1)
      end do
	v4(i) = bb(i,nm1)
      end do

	call multmm(l,r4,r6,r5)
	call multmv(l,r4,v4,v5)

      do i=1,l
      do j=1,l
	r5(i,j) = r5(i,j)+r3(i,j)
      end do
	v5(i) = v1(i)-v2(i)-v5(i)
      end do

	call inverse(l,r5)
	call multmv(l,r5,v5,v4)

      do i=1,l
        z(i,n) = v4(i)
      end do


c
c------------------------------------ the reverse 'progonka'
c
      do k = nm1,2,-1
       kp1 = k+1
      do i = 1,l
      do j = 1,l
       r1(i,j)   = aa(i,j,k)
      end do
        v1(i) = z(i,kp1)
      end do

        call multmv(l,r1,v1,v2)

      do i=1,l
        z(i,k) = v2(i)+bb(i,k)
      end do

      end do
c
c---------------------------------------- z_1= a_2^{-1}*[d_2-b_2*z_2-c_2*z_3]
c
      do i=1,l
      do j=1,l

        r1(i,j) = a(i,j,2)
	  r2(i,j) = b(i,j,2)
	  r3(i,j) = c(i,j,2)
      end do
	  v1(i) = d(i,2)
	  v2(i) = z(i,2)
	  v3(i) = z(i,3)
      end do

	  call inverse(l,r1)
	  call multmv(l,r2,v2,v4)
	  call multmv(l,r3,v3,v5)

      do i=1,l
	  v5(i) = v1(i) - v4(i) - v5(i)
      end do

	  call multmv(l,r1,v5,v1)

      do i=1,l
	  z(i,1) = v1(i)
      end do

	  return
        end
c
c*******************************************************************
c
      SUBROUTINE  MATIN2 (A,idim1,N1,idim2,N2,INDEX,NERROR,DETERM)
c     implicit real*8 (a-h,o-z)
C	 MATRIX INVERSION WITH ACCOMPANYING SOLUTION OF LINEAR EQUATIONS
      DOUBLE PRECISION A,DETERM,DETER,PIVOT,SWAP
      DIMENSION A(*),INDEX(*)
	idim2 = idim1
      DETER=1.0D0
      N=N1
      IEMAT=N+N2
      IDIM=IDIM1
      NMIN1=N-1
C	 THE ROUTINE DOES ITS OWN EVALUATION FOR DOUBLE SUBSCRIPTING OF
C	 ARRAY A.
      IPIVC=1-IDIM
C	 MAIN LOOP TO INVERT THE MATRIX
      DO 11 MAIN=1,N
      PIVOT=0.0D0
      IPIVC=IPIVC+IDIM
C	 SEARCH FOR NEXT PIVOT IN COLUMN MAIN.
      IPIVC1=IPIVC+MAIN-1
      IPIVC2=IPIVC +NMIN1
      DO 2 I1=IPIVC1,IPIVC2
      IF(DABS(A(I1))-DABS(PIVOT)) 2,2,1
    1 PIVOT=A(I1)
      LPIV=I1
    2 CONTINUE
C	 IS PIVOT DIFFERENT FROM ZERO
      IF(PIVOT) 3,15,3
C	 GET THE PIVOT-LINE INDICATOR AND SWAP LINES IF NECESSARY
    3 ICOL=LPIV-IPIVC+1
      INDEX(MAIN)=ICOL
      IF(ICOL-MAIN) 6,6,4
C	 COMPLEMENT THE DETERMINANT
    4 DETER=-DETER
C	 POINTER TO LINE PIVOT FOUND
      ICOL=ICOL-IDIM
C	 POINTER TO EXACT PIVOT LINE
      I3=MAIN-IDIM
      DO 5 I=1,IEMAT
      ICOL=ICOL+IDIM
      I3=I3+IDIM
      SWAP=A(I3)
      A(I3)=A(ICOL)
    5 A(ICOL)=SWAP
C	 COMPUTE DETERMINANT
    6 DETER=DETER*PIVOT
      PIVOT=1./PIVOT
C	 TRANSFORM PIVOT COLUMN
      I3=IPIVC+NMIN1
      DO 7 I=IPIVC,I3
    7 A(I)=-A(I)*PIVOT
      A(IPIVC1)=PIVOT
C	 PIVOT ELEMENT TRANSFORMED
C
C	 NOW CONVERT REST OF THE MATRIX
      I1=MAIN-IDIM
C	 POINTER TO PIVOT LINE ELEMENTS
      ICOL=1-IDIM
C	 GENERAL COLUMN POINTER
      DO 10 I=1,IEMAT
      ICOL=ICOL+IDIM
      I1=I1+IDIM
C	 POINTERS MOVED
      IF(I-MAIN) 8,10,8
C	 PIVOT COLUMN EXCLUDED
    8 JCOL=ICOL+NMIN1
      SWAP=A(I1)
      I3=IPIVC-1
      DO 9 I2=ICOL,JCOL
      I3=I3+1
    9 A(I2)=A(I2)+SWAP*A(I3)
      A(I1)=SWAP*PIVOT
   10 CONTINUE
   11 CONTINUE
C	 NOW REARRANGE THE MATRIX TO GET RIGHT INVERS
      DO 14 I1=1,N
      MAIN=N+1-I1
      LPIV=INDEX(MAIN)
      IF(LPIV-MAIN) 12,14,12
   12 ICOL=(LPIV-1)*IDIM+1
      JCOL=ICOL+NMIN1
      IPIVC=(MAIN-1)*IDIM+1-ICOL
      DO 13 I2=ICOL,JCOL
      I3=I2+IPIVC
      SWAP=A(I2)
      A(I2)=A(I3)
   13 A(I3)=SWAP
   14 CONTINUE
      DETERM=DETER
      NERROR=0
      RETURN
   15 NERROR=MAIN
      DETERM=DETER
      RETURN
      END


c*
c*******************************************************************
c
      subroutine inverse(n,a)
      implicit real*8 (a-h,o-z)
      dimension a(n,n)
      dimension index(100)

      call matin2(a,n,n,mdim,0,index,nerror,determ)

      return
      end

C
C*******************************************************************
C
      SUBROUTINE MULTMV(N,A,B,C)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(N,N),B(N),C(N)
      DO 1 I = 1,N
      C(I)   = 0.
      DO 1 J = 1,N
      C(I)   = C(I)+A(I,J)*B(J)
  1   CONTINUE
      RETURN
      END
C
C*******************************************************************
C
      SUBROUTINE MULTMM(N,A,B,C)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(N,N),B(N,N),C(N,N)
      DO 2 I = 1,N
      DO 2 J = 1,N
      C(I,J) = 0.
      DO 2 L = 1,N
      C(I,J) = C(I,J)+A(I,L)*B(L,J)
  2   CONTINUE
      RETURN
      END
c*